#include "stdafx.h"
#include "ClipQueue.h"

CClipQueue::CClipQueue(int am)
{
	deqpos = enqpos = 0;
	qsize = am + 1; // circular queue can contain less than size.
	selected = 0; // only for outside
}

CClipQueue::~CClipQueue()
{
	clear();
}

#define NEXT(x) { (x) = ((x)+1)%qsize; }

void CClipQueue::add(const char *str)
{
	slots[enqpos] = strdup(str);
	NEXT(enqpos);
	if (enqpos == deqpos) {
		free((void*)slots[deqpos]);
		NEXT(deqpos);
	}
	selected = 0;
}

void CClipQueue::clear()
{
	while (enqpos != deqpos) {
		free((void*)slots[deqpos]);
		NEXT(deqpos);
	}
	deqpos = enqpos = 0;
	selected = 0;
}

int CClipQueue::get_length()
{
	return (enqpos - deqpos + qsize) % qsize;
}

const char * CClipQueue::getitem(int pos)
{
	return slots[(deqpos + pos) % qsize];
}

void CClipQueue::resize(int newsize)
{
	const char *buf[MAX_SLOTS];
	int len=get_length(), i;

	for (i=0; i<len; i++) {
		buf[i] = getitem(i);
	}
	deqpos = enqpos = 0;
	qsize = newsize;
	selected = 0;
	for (i=0; i<len; i++) {
		slots[enqpos] = buf[i];
		NEXT(enqpos);
		if (enqpos == deqpos) {
			free((void*)slots[deqpos]);
			NEXT(deqpos);
		}
	}
}
